@@ -1,3 +1,6 @@
+#ifndef QT_FEATURES_Core_src_corelib_qtcore_config_p_h_H
+#define QT_FEATURES_Core_src_corelib_qtcore_config_p_h_H
+
 #define QT_FEATURE_clock_gettime 1
 
 #define QT_FEATURE_copy_file_range 1
@@ -26,7 +29,7 @@
 
 #define QT_FEATURE_jemalloc -1
 
-#define QT_FEATURE_journald -1
+#define QT_FEATURE_journald 1
 
 #define QT_FEATURE_system_libb2 1
 
@@ -104,3 +107,5 @@
 
 #define QT_FEATURE_openssl_hash -1
 
+
+#endif // QT_FEATURES_Core_src_corelib_qtcore_config_p_h_H
